use std::env;
use std::time::Duration;

use actix_identity::IdentityMiddleware;
use actix_session::SessionMiddleware;
use actix_web::cookie::Key;
use actix_web::{web, App, HttpServer};
use actix_web_static_files::ResourceFiles;
use brass_config::{get_env, load_config, Config};
use sqlx::postgres::PgPool;

use crate::postgres_session_store::SqlxPostgresqlSessionStore;
use crate::utils::manage_commands::{handle_command, parse_args};

mod auth;
mod endpoints;
mod middleware;
mod models;
mod utils;

mod filters;
mod postgres_session_store;

include!(concat!(env!("OUT_DIR"), "/generated.rs"));
include!(concat!(env!("OUT_DIR"), "/built.rs"));

#[actix_web::main]
async fn main() -> anyhow::Result<()> {
    let env = get_env()?;
    let config: Config = load_config(&env)?;

    let args = parse_args()?;

    let pool = PgPool::connect(&config.database_url).await?;
    let mailer = utils::email::get_mailer()?;
    let secret_key = Key::from(config.secret_key.as_bytes());
    let store = SqlxPostgresqlSessionStore::from_pool(pool.clone().into());

    handle_command(args.command, &pool).await?;

    println!("Starting server on http://{}:{}.", config.server_address, config.server_port);

    HttpServer::new(move || {
        let generated = generate();

        App::new()
            .app_data(web::Data::new(pool.clone()))
            .app_data(web::Data::new(mailer.clone()))
            .configure(endpoints::init)
            .wrap(middleware::RedirectToLogin)
            .wrap(middleware::LoadCurrentUser)
            .wrap(
                IdentityMiddleware::builder()
                    .visit_deadline(Some(Duration::from_secs(300)))
                    .build(),
            )
            .wrap(SessionMiddleware::new(store.clone(), secret_key.clone()))
            .service(ResourceFiles::new("/static", generated))
    })
    .bind((config.server_address, config.server_port))?
    .run()
    .await?;

    Ok(())
}
